import requests
import json
from errors import *


class Hypixel:
    def __init__(self, api_key):
        self.api_key = api_key

        # Hypixel API Key testing
        try:
            data = requests.get(f"https://api.hypixel.net/key?key={self.api_key}").json()
        except requests.exceptions.ConnectionError:
            raise ConnectionError("Failed to Connect to Hypixel API, try again later.")
        else:
            if not data["success"]:
                raise InvalidAPIKey(self.api_key)

    # Playername to UUID via Mojang API
    def getUUID(self, playername):
        try:
            data = requests.get(f"https://api.mojang.com/users/profiles/minecraft/{playername}").json()
        except json.decoder.JSONDecodeError:
            raise PlayerNotFound(playername)
        else:
            if "error" in data:
                if data["error"] == "BadRequestException":
                    raise InvalidPlayername(playername)
            else:
                return data["id"]

    # UUID to Playername via Mojang API
    def getPlayername(self, uuid):
        data = requests.get(f"https://sessionserver.mojang.com/session/minecraft/profile/{uuid}").json()
        if "errorMessage" in data:
            if "Not a valid UUID" in data["errorMessage"]:
                raise InvalidUUID(uuid)
        else:
            return data["name"]

    # SkyWars Stats Base Class
    class SkyWarsStats:
        def __init__(self, uuid, client):
            self.uuid = uuid
            self.__client = client
            self.__skywarsStats = self.__getSkyWarsStats(self.uuid)

            # Overall Stats
            self.overallWins = self.__skywarsStats["wins"] if 'wins' in self.__skywarsStats else None
            self.overallLosses = self.__skywarsStats["losses"] if 'losses' in self.__skywarsStats else None
            self.overallKills = self.__skywarsStats["kills"] if 'kills' in self.__skywarsStats else None
            self.overallMeleeKills = self.__skywarsStats[
                "melee_kills"] if 'melee_kills' in self.__skywarsStats else None
            self.overallEnderpearlsThrown = self.__skywarsStats[
                "enderpearls_thrown"] if 'enderpearls_thrown' in self.__skywarsStats else None
            self.overallDeaths = self.__skywarsStats["deaths"] if 'deaths' in self.__skywarsStats else None
            self.overallAssists = self.__skywarsStats["assists"] if 'assists' in self.__skywarsStats else None
            self.overallBowKills = self.__skywarsStats["bow_kills"] if 'bow_kills' in self.__skywarsStats else None
            self.overallVoidKills = self.__skywarsStats["void_kills"] if 'void_kills' in self.__skywarsStats else None
            self.overallEggsThrown = self.__skywarsStats[
                "overallEggsThrown"] if 'overallEggsThrown' in self.__skywarsStats else None
            self.overallBlocksPlaced = self.__skywarsStats[
                "blocks_placed"] if 'blocks_placed' in self.__skywarsStats else None
            self.overallGamesPlayed = self.__skywarsStats["games"] if 'games' in self.__skywarsStats else None
            self.overallHeads = self.__skywarsStats["heads"] if 'heads' in self.__skywarsStats else None
            self.coins = self.__skywarsStats["coins"] if 'coins' in self.__skywarsStats else None
            self.tokens = self.__skywarsStats["cosmetic_tokens"] if 'cosmetic_tokens' in self.__skywarsStats else None
            self.souls = self.__skywarsStats["souls"] if 'souls' in self.__skywarsStats else None

        def __getSkyWarsStats(self, uuid):
            try:
                data = requests.get(
                    f"https://api.hypixel.net/player?uuid={self.uuid}&key={self.__client.api_key}").json()
            except requests.exceptions.ConnectionError:
                raise ConnectionError("Failed to Connect to Hypixel API, try again later.")
            else:
                if not data["success"]:
                    if data["cause"] == "Malformed UUID":
                        raise InvalidUUID(uuid)
                    elif data["cause"] == "Invalid API key":
                        raise InvalidAPIKey(self.api_key)
                else:
                    if "stats" in data["player"]:
                        if "SkyWars" in data["player"]["stats"]:
                            return data["player"]["stats"]["SkyWars"]
                    else:
                        raise NoSkyWarsData(uuid)
